package com.aliyun.imp.flutter.sdk.alicloud_impinteraction_sdk.plugin;

import static com.aliyun.roompaas.chat.CommentSortType.TIME_ASC;
import static com.aliyun.roompaas.chat.CommentSortType.TIME_DESC;

import android.text.TextUtils;

import androidx.annotation.NonNull;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.aliyun.imp.flutter.sdk.alicloud_impinteraction_sdk.CallbackAdapter;
import com.aliyun.roompaas.chat.SampleChatEventHandler;
import com.aliyun.roompaas.chat.exposable.ChatService;
import com.aliyun.roompaas.chat.exposable.CommentParam;
import com.aliyun.roompaas.chat.exposable.event.CommentEvent;
import com.aliyun.roompaas.chat.exposable.event.CustomMessageEvent;
import com.aliyun.roompaas.chat.exposable.event.LikeEvent;
import com.aliyun.roompaas.chat.exposable.event.MuteAllCommentEvent;
import com.aliyun.roompaas.chat.exposable.event.MuteCommentEvent;

import java.lang.reflect.Type;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.Result;

public class FPOfChat extends FPOfBase {
    private ChatService chatService;

    private interface API {
        String getChatDetail = "getChatDetail";
        String sendComment = "sendComment";
        String sendCustomMessageToAll = "sendCustomMessageToAll";
        String sendCustomMessageToUsers = "sendCustomMessageToUsers";
        String banComment = "banComment";
        String cancelBanComment = "cancelBanComment";
        String banAllComment = "banAllComment";
        String cancelBanAllComment = "cancelBanAllComment";
        String sendLike = "sendLike";
        String listComment = "listComment";
    }

    @Override
    protected String getMethodChannelName() {
        return "alicloud_impinteraction_sdk_chat";
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        super.onMethodCall(call, result);
        switch (call.method) {
            case API.getChatDetail:
                getChatDetail(call, result);
                break;
            case API.sendComment:
                sendComment(call, result);
                break;
            case API.sendCustomMessageToAll:
                sendCustomMessageToAll(call, result);
                break;
            case API.sendCustomMessageToUsers:
                sendCustomMessageToUsers(call, result);
                break;
            case API.banComment:
                banComment(call, result);
                break;
            case API.cancelBanComment:
                cancelBanComment(call, result);
                break;
            case API.banAllComment:
                banAllComment(call, result);
                break;
            case API.cancelBanAllComment:
                cancelBanAllComment(call, result);
                break;
            case API.sendLike:
                sendLike(call, result);
                break;
            case API.listComment:
                listComment(call, result);
                break;
            default:
                result.notImplemented();
                break;
        }
    }

    private void getChatDetail(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        getChatService().getChatDetail(new CallbackAdapter<>(call, result));
    }

    private void sendComment(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        String content = call.argument("content");
        HashMap<String, String> extension = null;
        String extJson = call.argument("extension");
        if (!TextUtils.isEmpty(extJson)) {
            Type extType = new TypeReference<Map<String, String>>() {
            }.getType();
            Map<String, String> originMap = JSON.parseObject(extJson, extType);
            extension = new HashMap<>(originMap);
        }
        getChatService().sendComment(content, extension, new CallbackAdapter<>(call, result));
    }

    private void sendCustomMessageToAll(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        String body = call.argument("body");
        getChatService().sendCustomMessageToAll(body, new CallbackAdapter<>(call, result));
    }

    private void sendCustomMessageToUsers(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        String body = call.argument("body");
        List<String> userIds = null;
        String userIdsJsonArray = call.argument("userIds");
        if (userIdsJsonArray != null) {
            Type type = new TypeReference<List<String>>() {
            }.getType();
            userIds = JSON.parseObject(userIdsJsonArray, type);
        }
        getChatService().sendCustomMessageToUsers(body, userIds, new CallbackAdapter<>(call, result));
    }

    private void banComment(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        String userId = call.argument("userId");
        int muteSeconds = parseInt(call, "muteSeconds", 10);
        getChatService().banComment(userId, muteSeconds, new CallbackAdapter<>(call, result));
    }

    private void cancelBanComment(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        String userId = call.argument("userId");
        getChatService().cancelBanComment(userId, new CallbackAdapter<>(call, result));
    }

    private void banAllComment(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        getChatService().banAllComment(new CallbackAdapter<>(call, result));
    }

    private void cancelBanAllComment(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        getChatService().cancelBanAllComment(new CallbackAdapter<>(call, result));
    }

    private void sendLike(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        getChatService().sendLike();
        takeSuccess(result);
    }

    private void listComment(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        CommentParam param = new CommentParam();
        param.sortType = call.argument("sortType") == String.valueOf(TIME_ASC.ordinal()) ? TIME_ASC : TIME_DESC;
        getChatService().listComment(param, new CallbackAdapter<>(call, result));
    }

    private ChatService getChatService() {
        if (chatService == null || !Objects.equals(chatService, getRoomChannel().getPluginService(ChatService.class))) {
            chatService = getRoomChannel().getPluginService(ChatService.class);
            chatService.addEventHandler(new SampleChatEventHandler() {
                @Override
                public void onCommentReceived(CommentEvent event) {
                    postEvent("commentReceived", event);
                }

                @Override
                public void onCommentMutedOrCancel(MuteCommentEvent event) {
                    if (event.mute) {
                        postEvent("commentMuted", event);
                    } else {
                        postEvent("commentMuteCanceled", event);
                    }
                }

                @Override
                public void onCommentAllMutedOrCancel(MuteAllCommentEvent event) {
                    if (event.mute) {
                        postEvent("commentAllMuted", event);
                    } else {
                        postEvent("commentAllMuteCanceled", event);
                    }
                }

                @Override
                public void onLikeReceived(LikeEvent event) {
                    postEvent("liked", event);
                }

                @Override
                public void onCustomMessageReceived(CustomMessageEvent event) {
                    postEvent("customMessage", event);
                }
            });
        }
        return chatService;
    }

    @Override
    public void destroy() {
        super.destroy();
        chatService = null;
    }
}
