package com.aliyun.imp.flutter.sdk.alicloud_impinteraction_sdk.plugin;

import androidx.annotation.NonNull;

import com.aliyun.imp.flutter.sdk.alicloud_impinteraction_sdk.CallbackAdapter;
import com.aliyun.imp.flutter.sdk.alicloud_impinteraction_sdk.inner.Holder;
import com.aliyun.imp.flutter.sdk.alicloud_impinteraction_sdk.model.ModelAdapter;
import com.aliyun.roompaas.biz.RoomEngine;
import com.aliyun.roompaas.biz.SampleRoomEventHandler;
import com.aliyun.roompaas.biz.exposable.RoomChannel;
import com.aliyun.roompaas.biz.exposable.event.KickUserEvent;
import com.aliyun.roompaas.biz.exposable.event.RoomInOutEvent;
import com.aliyun.roompaas.biz.exposable.model.Result;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;

public class FPOfRoomChannel extends FPOfBase {

    private String roomId;

    private interface API {
        String getRoomId = "getRoomId";
        String isOwner = "isOwner";
        String isAdmin = "isAdmin";
        String setRoomId = "setRoomId";
        String enterRoom = "enterRoom";
        String leaveRoom = "leaveRoom";
        String listUser = "listUser";
        String getRoomDetail = "getRoomDetail";
        String kickUser = "kickUser";
    }

    @Override
    protected String getMethodChannelName() {
        return "alicloud_impinteraction_sdk_room_channel";
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        super.onMethodCall(call, result);
        switch (call.method) {
            case API.getRoomId:
                getRoomId(call, result);
                break;
            case API.isOwner:
                isOwner(call, result);
                break;
            case API.isAdmin:
                isAdmin(call, result);
                break;
            case API.setRoomId:
                setRoomId(call, result);
                break;
            case API.enterRoom:
                enterRoom(call, result);
                break;
            case API.leaveRoom:
                leaveRoom(call, result);
                break;
            case API.listUser:
                listUser(call, result);
                break;
            case API.getRoomDetail:
                getRoomDetail(call, result);
                break;
            case API.kickUser:
                kickUser(call, result);
                break;
            default:
                result.notImplemented();
                break;
        }
    }

    private void getRoomId(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        takeSuccess(result, roomId);
    }

    private void isOwner(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        takeSuccess(result, getRoomChannel().isOwner());
    }

    private void isAdmin(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        takeSuccess(result, getRoomChannel().isAdmin());
    }

    private void setRoomId(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        String roomId = call.argument("roomId");
        this.roomId = roomId;

        Result<RoomChannel> res = RoomEngine.getInstance().getRoomChannel(roomId);
        Holder.INSTANCE.roomChannel = res.value;
        takeSuccess(result);
    }

    private void enterRoom(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        getRoomChannel().enterRoom(call.argument("nick"), call.argument("extension"), new CallbackAdapter<>(call, result));
    }

    private void leaveRoom(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        getRoomChannel().leaveRoom(new CallbackAdapter<Void>(call, result) {
            @Override
            public void onSuccess(Void data) {
                super.onSuccess(data);
                // 离开房间时释放RoomChannel
                Holder.INSTANCE.roomChannel = null;
            }
        });
    }

    private void listUser(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        int pageNum = parseInt(call, "pageNum", 1);
        int pageSize = parseInt(call, "pageSize", 10);
        getRoomChannel().listUser(ModelAdapter.ofUserParam(pageNum, pageSize), new CallbackAdapter<>(call, result, true));
    }

    private void getRoomDetail(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        getRoomChannel().getRoomDetail(new CallbackAdapter<>(call, result, true));
    }

    private void kickUser(@NonNull MethodCall call, @NonNull MethodChannel.Result result) {
        String userId = call.argument("userId");
        int kickedSeconds = parseInt(call, "kickedSeconds", 0);
        getRoomChannel().kickUser(userId, kickedSeconds, new CallbackAdapter<>(call, result));
    }

    protected RoomChannel getRoomChannel() {
        RoomChannel roomChannel = Holder.INSTANCE.roomChannel;
        if (roomChannel != null) {
            return roomChannel;
        }

        Result<RoomChannel> result = RoomEngine.getInstance().getRoomChannel(roomId);
        if (result.success) {
            RoomChannel channel = result.value;
            Holder.INSTANCE.roomChannel = channel;
            channel.addEventHandler(new SampleRoomEventHandler() {
                @Override
                public void onEnterOrLeaveRoom(RoomInOutEvent event) {
                    if (event.enter) {
                        postEvent("enterRoom", event);
                    } else {
                        postEvent("leaveRoom", event);
                    }
                }

                @Override
                public void onRoomUserKicked(KickUserEvent event) {
                    postEvent("kickedFromRoom", event);
                }
            });
            return channel;
        } else {
            throw new RuntimeException("Get room channel fail: " + result.errorMsg);
        }
    }
}
