package com.aliyun.imp.flutter.sdk.alicloud_impinteraction_sdk.plugin;

import android.app.Activity;
import android.text.TextUtils;
import android.util.Log;

import androidx.annotation.NonNull;

import com.aliyun.imp.flutter.sdk.alicloud_impinteraction_sdk.CallbackAdapter;
import com.aliyun.imp.flutter.sdk.alicloud_impinteraction_sdk.NativeViewFactory;
import com.aliyun.imp.flutter.sdk.alicloud_impinteraction_sdk.inner.Holder;
import com.aliyun.imp.flutter.sdk.alicloud_impinteraction_sdk.model.InitParam;
import com.aliyun.roompaas.base.exposable.Callback;
import com.aliyun.roompaas.base.log.ComposeLoggerHandler;
import com.aliyun.roompaas.base.log.DefaultLoggerHandler;
import com.aliyun.roompaas.base.log.FileLoggerHandler;
import com.aliyun.roompaas.base.log.Logger;
import com.aliyun.roompaas.base.util.CommonUtil;
import com.aliyun.roompaas.base.util.Utils;
import com.aliyun.roompaas.biz.EngineConfig;
import com.aliyun.roompaas.biz.RoomEngine;
import com.aliyun.roompaas.biz.exposable.IRoomEngineEventHandler;
import com.aliyun.roompaas.biz.exposable.model.TokenInfo;
import com.aliyun.roompaas.uibase.util.AppUtil;

import io.flutter.embedding.engine.FlutterEngine;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel.Result;

public class FPOfRoomEngine extends FPOfBase {
    private static final String EVENT_REQUEST_TOKEN = "onRequestToken";
    private static final String EVENT_KICK = "onKicked";
    private static final String NATIVE_VIEW_TAG = "<platform-view-type>";

    private InitParam initParam;
    private Callback<TokenInfo> tokenGetterCallback;

    private interface API {
        String init = "init";
        String login = "login";
        String setLoginToken = "setLoginToken";
        String logout = "logout";
        String isLogined = "isLogined";
    }

    public FPOfRoomEngine() {
        super();
        // 监听kick事件
        RoomEngine.getInstance().setEventHandler(event -> {
            if (event == IRoomEngineEventHandler.EngineEvent.KICK_OUT) {
                postEvent(EVENT_KICK);
            }
        });
    }

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding binding) {
        super.onAttachedToEngine(binding);
        FlutterEngine flutterEngine = binding.getFlutterEngine();
        flutterEngine
                .getPlatformViewsController()
                .getRegistry()
                .registerViewFactory(NATIVE_VIEW_TAG, new NativeViewFactory());
    }

    @Override
    protected String getMethodChannelName() {
        return "alicloud_impinteraction_sdk_room_engine";
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        super.onMethodCall(call, result);
        switch (call.method) {
            case API.init:
                init(call, result);
                break;
            case API.login:
                login(call, result);
                break;
            case API.setLoginToken:
                setLoginToken(call, result);
                break;
            case API.logout:
                logout(call, result);
                break;
            case API.isLogined:
                isLogined(call, result);
                break;
            default:
                result.notImplemented();
                break;
        }
    }

    private void init(@NonNull MethodCall call, @NonNull Result result) {
        initParam = new InitParam();
        initParam.userId = call.argument("userId");
        initParam.appId = call.argument("appId");
        initParam.appKey = call.argument("appKey4Android");
        initParam.serverHost = call.argument("serverHost");
        initParam.serverSecret = call.argument("serverSecret");
        Holder.INSTANCE.param = initParam;
        Holder.INSTANCE.deviceId = Utils.acceptFirstNotEmpty(call.argument("deviceId"), CommonUtil.getDeviceId());

        Log.i(TAG, "init: " + initParam);
        Activity activity = Utils.getRef(activityRef);
        if (Utils.isActivityInvalid(activity)) {
            Log.i(TAG, "end : invalid activity:" + activityRef);
            takeErrOfLifecycle(result, "Android init: ");
        } else {
            RoomEngine.getInstance().init(activity,
                    new EngineConfig.Builder()
                            .appId(initParam.appId)
                            .appKey(initParam.appKey)
                            .deviceId(Holder.INSTANCE.deviceId)
                            .loggerHandler(new ComposeLoggerHandler(
                                    new DefaultLoggerHandler(),
                                    new FileLoggerHandler(AppUtil.getAppContext())
                            ))
                            .tokenInfoGetter((userId, callback) -> {
                                tokenGetterCallback = callback;
                                Logger.i(TAG, "init: tokenGetterCallback=" + tokenGetterCallback);
                                postEvent(EVENT_REQUEST_TOKEN);
                            })
                            .build(),
                    new CallbackAdapter<>(call, result)
            );
        }
    }

    public void login(@NonNull MethodCall call, @NonNull Result result) {
        RoomEngine.getInstance().auth(initParam.userId, new CallbackAdapter<>(call, result));
    }

    private void setLoginToken(@NonNull MethodCall call, @NonNull Result result) {
        String accessToken = call.argument("accessToken");
        String refreshToken = call.argument("refreshToken");
        if (tokenGetterCallback == null) {
            String msg = "setLoginToken: no callback found.";
            Logger.w(TAG, msg);
            takeErr(result, msg);
            return;
        }

        if (TextUtils.isEmpty(refreshToken) || TextUtils.isEmpty(accessToken)) {
            String msg = "Get token fail: refreshToken and accessToken can either be empty";
            tokenGetterCallback.onError(msg);
            takeErr(result, msg);
        } else {
            TokenInfo tokenInfo = new TokenInfo();
            tokenInfo.accessToken = accessToken;
            tokenInfo.refreshToken = refreshToken;
            tokenGetterCallback.onSuccess(tokenInfo);
            takeSuccess(result);
        }
        // 使用后即释放
        tokenGetterCallback = null;
    }

    private void logout(@NonNull MethodCall call, @NonNull Result result) {
        RoomEngine.getInstance().logout(new CallbackAdapter<>(call, result));
    }

    private void isLogined(@NonNull MethodCall call, @NonNull Result result) {
        result.success(asSimpleResult(RoomEngine.getInstance().isLogin()));
    }
}
