#import "IMPChatModule.h"
#import "IMPRoomChannelManager.h"
#import "AIMPSConf.h"

@import AliInteractiveRoomBundle;
#import "IMPEvent.h"

#define CHAT [IMPRoomChannelManager sharedInstance].room.chat

@implementation IMPChatModule

#pragma mark -Lifecycle

- (instancetype)init {
    self = [super init];
    if (self) {
        __weak typeof(self) weakSelf = self;
        [IMPRoomChannelManager sharedInstance].onReceiveChatMessage = ^(AIRBRoomChannelMessageType type, NSDictionary *_Nonnull info) {
            [weakSelf p_handleMessage:type info:info];
        };
    }
    return self;
}

- (void)handleMethodCall:(FlutterMethodCall *)call result:(FlutterResult)result {
    NSString *apiName = call.method;
    NSLog([NSString stringWithFormat:@"iOS api invoked: %@", apiName]);

    if ([@"getChatDetail" isEqualToString:apiName]) {
        [self getChatDetail:call result:result];
    } else if ([@"sendComment" isEqualToString:apiName]) {
        [self sendComment:call result:result];
    } else if ([@"sendCustomMessageToAll" isEqualToString:apiName]) {
        [self sendCustomMessageToAll:call result:result];
    } else if ([@"sendCustomMessageToUsers" isEqualToString:apiName]) {
        [self sendCustomMessageToUsers:call result:result];
    } else if ([@"banComment" isEqualToString:apiName]) {
        [self banComment:call result:result];
    } else if ([@"cancelBanComment" isEqualToString:apiName]) {
        [self cancelBanComment:call result:result];
    } else if ([@"banAllComment" isEqualToString:apiName]) {
        [self banAllComment:call result:result];
    } else if ([@"cancelBanAllComment" isEqualToString:apiName]) {
        [self cancelBanAllComment:call result:result];
    } else if ([@"sendLike" isEqualToString:apiName]) {
        [self sendLike:call result:result];
    } else if ([@"listComment" isEqualToString:apiName]) {
        [self listComment:call result:result];
    } else {
        result(FlutterMethodNotImplemented);
    }
}

- (void)getChatDetail:(FlutterMethodCall *)call result:(FlutterResult)result {
    [CHAT getCurrentChatInfoOnSuccess:^(NSDictionary *_Nonnull info) {
        NSMutableDictionary *contentDictionary = [@{
                @"likeCount": info[@"total_like"],
                @"commentCount": info[@"total_comment"],
                @"mute": info[@"ban"],
                @"muteAll": info[@"ban_all"]
        } mutableCopy];
        [AIMPSUtils take:result dic:contentDictionary];
    }                       onFailure:^(NSString *_Nonnull errMessage) {
        [AIMPSUtils callErr:result msg:errMessage];
    }];
}

- (void)sendComment:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (call.arguments[@"content"]) {
        NSDictionary *extension = nil;
        if (call.arguments[@"extension"] && [call.arguments[@"extension"] isKindOfClass:[NSDictionary class]]) {
            extension = call.arguments[@"extension"];
        }
        [CHAT sendComment:call.arguments[@"content"] extension:extension onSuccess:^{
            [AIMPSUtils callOK:result];
        }       onFailure:^(AIRBErrorCode code, NSString *_Nonnull message) {
            [AIMPSUtils callErr:result msg:message];
        }];
    } else {
        [AIMPSUtils callErr:result msg:[AIMPSUtils wrapInvalidParam:@"content"]];
    }
}

- (void)sendCustomMessageToAll:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (call.arguments[@"body"]) {
        [CHAT sendCustomMessageToALL:call.arguments[@"body"] onSuccess:^{
            [AIMPSUtils callOK:result];
        }                  onFailure:^(NSString *_Nonnull errorMessage) {
            [AIMPSUtils callErr:result msg:errorMessage];
        }];
    } else {
        [AIMPSUtils callErr:result msg:[AIMPSUtils wrapInvalidParam:@"body"]];
    }
}

- (void)sendCustomMessageToUsers:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (call.arguments[@"body"] && call.arguments[@"userIds"] && [call.arguments[@"userIds"] isKindOfClass:[NSArray class]]) {
        [CHAT sendCustomMessage:call.arguments[@"body"] toUsers:call.arguments[@"userIds"] onSuccess:^{
            [AIMPSUtils callOK:result];
        }             onFailure:^(NSString *_Nonnull errorMessage) {
            [AIMPSUtils callErr:result msg:errorMessage];
        }];
    } else {
        [AIMPSUtils callErr:result msg:[AIMPSUtils wrapInvalidParam:@"body&userIds"]];
    }
}

- (void)banComment:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (call.arguments[@"userId"]) {
        int32_t banSeconds = 0;
        if (call.arguments[@"muteSeconds"]) {
            banSeconds = [call.arguments[@"muteSeconds"] intValue];
        }
        [CHAT banCommentWithUserID:call.arguments[@"userId"] banTimeInSeconds:banSeconds onSuccess:^{
            [AIMPSUtils callOK:result];
        }                onFailure:^(NSString *_Nonnull errorMessage) {
            [AIMPSUtils callErr:result msg:errorMessage];
        }];
    } else {
        [AIMPSUtils callErr:result msg:[AIMPSUtils wrapInvalidParam:@"userId"]];
    }
}

- (void)cancelBanComment:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (call.arguments[@"userId"]) {
        [CHAT cancelBanCommentWithUserID:call.arguments[@"userId"] onSuccess:^{
            [AIMPSUtils callOK:result];
        }                      onFailure:^(NSString *_Nonnull errorMessage) {
            [AIMPSUtils callErr:result msg:errorMessage];
        }];
    } else {
        [AIMPSUtils callErr:result msg:[AIMPSUtils wrapInvalidParam:@"userId"]];
    }
}

- (void)banAllComment:(FlutterMethodCall *)call result:(FlutterResult)result {
    [CHAT banAllComment:YES onSuccess:^{
        [AIMPSUtils callOK:result];
    }         onFailure:^(NSString *_Nonnull errorMessage) {
        [AIMPSUtils callErr:result msg:errorMessage];
    }];
}

- (void)cancelBanAllComment:(FlutterMethodCall *)call result:(FlutterResult)result {
    [CHAT banAllComment:NO onSuccess:^{
        [AIMPSUtils callOK:result];
    }         onFailure:^(NSString *_Nonnull errorMessage) {
        [AIMPSUtils callErr:result msg:errorMessage];
    }];
}

- (void)sendLike:(FlutterMethodCall *)call result:(FlutterResult)result {
    [CHAT sendLike];
    [AIMPSUtils callOK:result];
}

- (void)listComment:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (call.arguments[@"pageNum"] && call.arguments[@"pageSize"] && call.arguments[@"sortType"]) {
        // TIME_ASC: 按时间升序; TIME_DESC: 按时间降序;
        AIRBRoomChatCommentsSortedType sortType = AIRBRoomChatCommentsSortedTypeTimestampAscending;
        if ([call.arguments[@"sortType"] isEqualToString:@"TIME_DESC"]) {
            sortType = AIRBRoomChatCommentsSortedTypeTimestampDescending;
        }
        [CHAT queryCommentsWithSortedType:sortType pageNum:[call.arguments[@"pageNum"] intValue] pageSize:[call.arguments[@"pageSize"] intValue] onSuccess:^(AIRBRoomChannelCommentsResponse *_Nonnull response) {
            NSMutableDictionary *contentDictionary = [@{
                    @"total": @(response.total),
                    @"hasMore": @(response.hasMore),
                    @"list": response.commentList
            } mutableCopy];
            [AIMPSUtils take:result dic:contentDictionary];
        }                       onFailure:^(NSString *_Nonnull errorMsg) {
            [AIMPSUtils callErr:result msg:errorMsg];
        }];
    } else {
        [AIMPSUtils callErr:result msg:[AIMPSUtils wrapInvalidParam:@"pageNum&pageSize&sortType"]];
    }
}

#pragma mark -Private

- (void)p_handleMessage:(AIRBRoomChannelMessageType)type info:(NSDictionary *)info {
    NSString *eventName = nil;
    switch (type) {
        case AIRBRoomChannelMessageTypeChatLikeReceived: {
            eventName = @"liked";
        }
            break;
        case AIRBRoomChannelMessageTypeChatCommentReceived: {
            eventName = @"commentReceived";
        }
            break;
        case AIRBRoomChannelMessageTypeChatCustomMessageReceived: {
            eventName = @"customMessage";
        }
            break;
        case AIRBRoomChannelMessageTypeChatOneUserCommentBannedOrNot: {
            if ([info[@"mute"] boolValue]) {
                eventName = @"commentMuted";
            } else {
                eventName = @"commentMuteCanceled";
            }
        }
            break;
        case AIRBRoomChannelMessageTypeChatAllUsersCommentBannedOrNot: {
            if ([info[@"mute"] boolValue]) {
                eventName = @"commentAllMuted";
            } else {
                eventName = @"commentAllMuteCanceled";
            }
        }
            break;
        default:
            break;
    }
    [IMPEvent fireEvent:eventName params:info];
}

@end
